@@ -49,10 +49,14 @@
  *      acting in its behalf permission to use and distribute the
  *      software in accordance with the terms specified in this
  *      license.
+ *
+ * RCS: @(#) $Id: tkMacOSXDebug.c,v 1.2.2.4 2005/11/27 02:36:46 das Exp $
  */
 
 #include "tkMacOSXDebug.h"
 
+#ifdef TK_MAC_DEBUG
+
 typedef struct {
  EventKind kind;
  char     * name;
@@ -226,7 +230,7 @@ static MyEventNameList eventNameList [] = {
 };
 
 
-MyEventName classicEventNames [] = {
+static MyEventName classicEventNames [] = {
  { nullEvent,"nullEvent" },
  { mouseDown,"mouseDown" },
  { mouseUp,"mouseUp" },
@@ -285,7 +289,8 @@ CarbonEventToAscii(EventRef eventRef, char * buf)
     return iBuf;
 }
 
-char * CarbonEventKindToAscii(EventRef eventRef, char * buf )
+char *
+CarbonEventKindToAscii(EventRef eventRef, char * buf )
 {     
    EventClass eventClass;
    EventKind  eventKind;
@@ -320,7 +325,8 @@ char * CarbonEventKindToAscii(EventRef eventRef, char * buf )
      return buf;
 }
 
-char * ClassicEventToAscii(EventRecord * eventPtr, char * buf )
+char *
+ClassicEventToAscii(EventRecord * eventPtr, char * buf )
 {
     MyEventName     * names = NULL;
     int found = 0;
@@ -349,27 +355,31 @@ char * ClassicEventToAscii(EventRecord * eventPtr, char * buf )
  
 }
 
-void printPoint(char * tag, Point * p )
+void
+printPoint(char * tag, Point * p )
 {
     fprintf(stderr,"%s %4d %4d\n",
         tag,p->h,p->v );
 }
 
-void printRect(char * tag, Rect * r )
+void
+printRect(char * tag, Rect * r )
 {
     fprintf(stderr,"%s %4d %4d %4d %4d (%dx%d)\n",
         tag, r->left, r->top, r->right, r->bottom,
         r->right - r->left + 1, r->bottom - r->top + 1);
 }
 
-void printRegion(char * tag, RgnHandle rgn )
+void
+printRegion(char * tag, RgnHandle rgn )
 {
     Rect r;
     GetRegionBounds(rgn,&r);
     printRect(tag,&r);
 }
 
-void printWindowTitle(char * tag, WindowRef window )
+void
+printWindowTitle(char * tag, WindowRef window )
 {
     Str255 title;
     GetWTitle(window,title);
@@ -382,7 +392,7 @@ typedef struct {
  char * name;
 } MsgName;
 
-MsgName msgNames [] = {
+static MsgName msgNames [] = {
     { kMenuDrawMsg,       "Draw"},
     { kMenuSizeMsg,       "Size"},
     { kMenuPopUpMsg,      "PopUp"},
@@ -396,7 +406,6 @@ MsgName msgNames [] = {
     { -1, NULL }
 };
 
-
 char *
 TkMacOSXMenuMessageToAscii(int msg, char * s)
 {
@@ -413,20 +422,21 @@ TkMacOSXMenuMessageToAscii(int msg, char * s)
     return s;
 }
 
+static MsgName trackingNames [] = {
+    { kMouseTrackingMousePressed  , "MousePressed  " },
+    { kMouseTrackingMouseReleased , "MouseReleased " },
+    { kMouseTrackingMouseExited   , "MouseExited   " },
+    { kMouseTrackingMouseEntered  , "MouseEntered  " },
+    { kMouseTrackingMouseMoved    , "MouseMoved    " },
+    { kMouseTrackingKeyModifiersChanged, "KeyModifiersChanged" },
+    { kMouseTrackingUserCancelled , "UserCancelled " },
+    { kMouseTrackingTimedOut      , "TimedOut      " },
+    { -1, NULL }
+};
 
-char * MouseTrackingResultToAscii(MouseTrackingResult r, char * buf) 
+char *
+MouseTrackingResultToAscii(MouseTrackingResult r, char * buf) 
 {
-    MsgName trackingNames [] = {
-        { kMouseTrackingMousePressed  , "MousePressed  " },
-        { kMouseTrackingMouseReleased , "MouseReleased " },
-        { kMouseTrackingMouseExited   , "MouseExited   " },
-        { kMouseTrackingMouseEntered  , "MouseEntered  " },
-        { kMouseTrackingMouseMoved    , "MouseMoved    " },
-        { kMouseTrackingKeyModifiersChanged, "KeyModifiersChanged" },
-        { kMouseTrackingUserCancelled , "UserCancelled " },
-        { kMouseTrackingTimedOut      , "TimedOut      " },
-        { -1, NULL }
-    };
     MsgName * namePtr;
     for (namePtr = trackingNames; namePtr->name; namePtr++) {
         if (namePtr->msg == r) {
@@ -437,3 +447,5 @@ char * MouseTrackingResultToAscii(MouseTrackingResult r, char * buf)
     sprintf(buf, "Unknown mouse tracking result : %d", r);
     return buf;
 }
+
+#endif /* TK_MAC_DEBUG */
