@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXDialog.c,v 1.4.2.4 2005/05/14 20:53:31 das Exp $
+ * RCS: @(#) $Id: tkMacOSXDialog.c,v 1.4.2.9 2005/11/27 02:36:46 das Exp $
  */
 #include <Carbon/Carbon.h>
 
@@ -19,10 +19,10 @@
 #include "tkFileFilter.h"
 
 #ifndef StrLength
-#define StrLength(s)                 (*((unsigned char *) (s)))
+#define StrLength(s)	(*((unsigned char *) (s)))
 #endif
 #ifndef StrBody
-#define StrBody(s)                ((char *) (s) + 1)
+#define StrBody(s)	((char *) (s) + 1)
 #endif
 
 /*
@@ -47,14 +47,12 @@
  * information about the file dialog and the file filters.
  */
 typedef struct _OpenFileData {
-    FileFilterList fl;                        /* List of file filters. */
-    SInt16 curType;                        /* The filetype currently being
-                                         * listed. */
-    short popupItem;                        /* Item number of the popup in the
-                                         * dialog. */
-    int usePopup;                        /* True if we show the popup menu (this
-                                             * is an open operation and the
-                                         * -filetypes option is set). */
+    FileFilterList fl;		/* List of file filters. */
+    SInt16 curType;		/* The filetype currently being listed. */
+    short popupItem;		/* Item number of the popup in the dialog. */
+    int usePopup;		/* True if we show the popup menu (this is
+				 * an open operation and the -filetypes
+				 * option is set). */
 } OpenFileData;
 
 
@@ -75,7 +73,7 @@ static Boolean                MatchOneType _ANSI_ARGS_((StringPtr fileNamePtr, O
 static pascal Boolean   OpenFileFilterProc(AEDesc* theItem, void* info, 
                             NavCallBackUserData callBackUD,
                             NavFilterModes filterMode );
-pascal void             OpenEventProc(NavEventCallbackMessage callBackSelector,
+static pascal void      OpenEventProc(NavEventCallbackMessage callBackSelector,
                             NavCBRecPtr callBackParms,
                             NavCallBackUserData callBackUD );
 static void             InitFileDialogs();
@@ -87,7 +85,7 @@ static int              HandleInitialDirectory (Tcl_Interp *interp,
                                                 char *initialFile, char *initialDir,
                                                 FSRef *dirRef,
                                                 AEDescList *selectDescPtr,
-                                                AEDesc *dirDescPtr);                            
+                                                AEDesc *dirDescPtr);
 
 /*
  * Have we initialized the file dialog subsystem
@@ -100,8 +98,8 @@ static int fileDlgInited = 0;
  * commands.
  */
 
-NavObjectFilterUPP openFileFilterUPP;
-NavEventUPP openFileEventUPP;
+static NavObjectFilterUPP openFileFilterUPP;
+static NavEventUPP openFileEventUPP;
 
 
 /*
@@ -273,15 +271,15 @@ Tk_GetOpenFileObjCmd(
     AEDescList selectDesc = {typeNull, NULL};
     char *initialFile = NULL, *initialDir = NULL;
     static CONST char *openOptionStrings[] = {
-            "-defaultextension", "-filetypes", 
-            "-initialdir", "-initialfile", 
+            "-defaultextension", "-filetypes",
+            "-initialdir", "-initialfile",
             "-message", "-multiple",
-            "-parent",        "-title",         NULL
+            "-parent", "-title", NULL
     };
     enum openOptions {
-            OPEN_DEFAULT, OPEN_FILETYPES,        
+            OPEN_DEFAULT, OPEN_FILETYPES,
             OPEN_INITDIR, OPEN_INITFILE,
-            OPEN_MESSAGE, OPEN_MULTIPLE, 
+            OPEN_MESSAGE, OPEN_MULTIPLE,
             OPEN_PARENT, OPEN_TITLE
     };
     
@@ -289,14 +287,14 @@ Tk_GetOpenFileObjCmd(
         InitFileDialogs();
     }
     
-    result = TCL_ERROR;    
-    parent = (Tk_Window) clientData; 
+    result = TCL_ERROR;
+    parent = (Tk_Window) clientData;
     multiple = false;
     title = NULL;
-    message = NULL;   
+    message = NULL;
 
     TkInitFileFilters(&ofd.fl);
-    
+
     ofd.curType                = 0;
     ofd.popupItem        = OPEN_POPUP_ITEM;
     ofd.usePopup         = 1;
@@ -331,10 +329,14 @@ Tk_GetOpenFileObjCmd(
                 }
                 break;
             case OPEN_INITDIR:
-                initialDir = Tcl_GetStringFromObj(objv[i + 1], NULL);
+                initialDir = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		/* empty strings should be like no selection given */
+		if (choiceLen == 0) { initialDir = NULL; }
                 break;
             case OPEN_INITFILE:
                 initialFile = Tcl_GetStringFromObj(objv[i + 1], NULL);
+		/* empty strings should be like no selection given */
+		if (choiceLen == 0) { initialFile = NULL; }
                 break;
             case OPEN_MESSAGE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
@@ -374,8 +376,7 @@ Tk_GetOpenFileObjCmd(
         initialPtr = &initialDesc;
     }
     result = NavServicesGetFile(interp, &ofd, initialPtr,
-            NULL, &selectDesc, 
-            title, message, multiple, OPEN_FILE);
+            NULL, &selectDesc, title, message, multiple, OPEN_FILE);
 
     end:
     TkFreeFileFilters(&ofd.fl);
@@ -434,11 +435,11 @@ Tk_GetSaveFileObjCmd(
     if (!fileDlgInited) {
         InitFileDialogs();
     }
-    
-    result = TCL_ERROR;    
-    parent = (Tk_Window) clientData;    
+
+    result = TCL_ERROR;
+    parent = (Tk_Window) clientData;
     title = NULL;
-    message = NULL;   
+    message = NULL;
 
     for (i = 1; i < objc; i += 2) {
         char *choice;
@@ -462,15 +463,19 @@ Tk_GetSaveFileObjCmd(
                 /* Currently unimplemented - what would we do here anyway? */
                 break;
             case SAVE_INITDIR:
-                choice = Tcl_GetStringFromObj(objv[i + 1], NULL);
-                if (HandleInitialDirectory(interp, NULL, choice, &dirRef, 
-                        NULL, &initialDesc) != TCL_OK) {
-                    result = TCL_ERROR;
-                    goto end;
-                }
+                choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		/* empty strings should be like no selection given */
+		if (choiceLen &&
+			HandleInitialDirectory(interp, NULL, choice, &dirRef, 
+				NULL, &initialDesc) != TCL_OK) {
+		    result = TCL_ERROR;
+		    goto end;
+		}
                 break;
             case SAVE_INITFILE:
-                initialFile = Tcl_GetStringFromObj(objv[i + 1], NULL);
+                initialFile = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		/* empty strings should be like no selection given */
+		if (choiceLen == 0) { initialFile = NULL; }
                 break;
             case SAVE_MESSAGE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
@@ -492,7 +497,7 @@ Tk_GetSaveFileObjCmd(
                 break;
         }
     }
-         
+
     TkInitFileFilters(&ofd.fl);
     ofd.usePopup = 0;
 
@@ -503,7 +508,7 @@ Tk_GetSaveFileObjCmd(
         title, message, false, SAVE_FILE);
 
     end:
-    
+
     AEDisposeDesc(&initialDesc);
     if (title != NULL) {
         CFRelease(title);
@@ -511,7 +516,7 @@ Tk_GetSaveFileObjCmd(
     if (message != NULL) {
         CFRelease(message);
     }
-    
+
     return result;
 }
 
@@ -549,14 +554,13 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
     CFStringRef message, title;
     OpenFileData ofd;
     static CONST char *chooseOptionStrings[] = {
-            "-initialdir", "-message", "-mustexist", "-parent", "-title", NULL
+	"-initialdir", "-message", "-mustexist", "-parent", "-title", NULL
     };
     enum chooseOptions {
-            CHOOSE_INITDIR,        CHOOSE_MESSAGE, CHOOSE_MUSTEXIST, 
-            CHOOSE_PARENT, CHOOSE_TITLE
+	CHOOSE_INITDIR, CHOOSE_MESSAGE, CHOOSE_MUSTEXIST,
+	CHOOSE_PARENT, CHOOSE_TITLE
     };
-  
-    
+
     if (!NavServicesAvailable()) {
         return TCL_ERROR;
     }
@@ -564,10 +568,10 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
     if (!fileDlgInited) {
         InitFileDialogs();
     }
-    result = TCL_ERROR;    
-    parent = (Tk_Window) clientData;    
+    result = TCL_ERROR;
+    parent = (Tk_Window) clientData;
     title = NULL;
-    message = NULL;   
+    message = NULL;
 
     for (i = 1; i < objc; i += 2) {
         char *choice;
@@ -586,9 +590,10 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
         }
         switch (index) {
             case CHOOSE_INITDIR:
-                choice = Tcl_GetStringFromObj(objv[i + 1], NULL);
-                if (HandleInitialDirectory(interp, NULL, choice, &dirRef,  
-                        NULL, &initialDesc) != TCL_OK) {
+                choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
+		if (choiceLen &&
+			HandleInitialDirectory(interp, NULL, choice, &dirRef,
+				NULL, &initialDesc) != TCL_OK) {
                     result = TCL_ERROR;
                     goto end;
                 }
@@ -613,11 +618,10 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
                 break;
         }
     }
-             
+
     TkInitFileFilters(&ofd.fl);
     ofd.usePopup = 0;
 
-        
     if (initialDesc.descriptorType == typeFSRef) {
         initialPtr = &initialDesc;
     }
@@ -658,7 +662,7 @@ HandleInitialDirectory (
         }
 
         err = FSPathMakeRef((unsigned char*) dirName,
-                dirRef, &isDirectory);     
+                dirRef, &isDirectory);
 
         if (err != noErr) {
             Tcl_AppendResult(interp, "bad directory \"",
@@ -749,24 +753,24 @@ NavServicesGetFile(
         diagOptions.optionFlags += kNavAllowMultipleFiles;
     }
     diagOptions.modality = kWindowModalityAppModal;
-            
+
     if (ofdPtr != NULL && ofdPtr->usePopup) {
         FileFilter *filterPtr;
-        
+
         filterPtr = ofdPtr->fl.filters;
         if (filterPtr == NULL) {
             ofdPtr->usePopup = 0;
         }
     }
-    
-    if (ofdPtr != NULL && ofdPtr->usePopup) {    
+
+    if (ofdPtr != NULL && ofdPtr->usePopup) {
         FileFilter *filterPtr;
         int index = 0;
         ofdPtr->curType = 0;
-        
+
         menuItemNames = (CFStringRef *)ckalloc(ofdPtr->fl.numFilters 
             * sizeof(CFStringRef));
-        
+
         for (filterPtr = ofdPtr->fl.filters; filterPtr != NULL; 
                 filterPtr = filterPtr->next, index++) {
             menuItemNames[index] = CFStringCreateWithCString(NULL, 
@@ -774,7 +778,7 @@ NavServicesGetFile(
         }
         diagOptions.popupExtension = CFArrayCreate(NULL, 
                 (const void **) menuItemNames, ofdPtr->fl.numFilters, NULL);
-    } else {        
+    } else {
         diagOptions.optionFlags += kNavNoTypePopup; 
         diagOptions.popupExtension = NULL;
     }
@@ -814,21 +818,27 @@ NavServicesGetFile(
             ofdPtr,
             &dialogRef);
         if (err != noErr) {
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"NavCreateGetFileDialog failed, %d\n", err );
+#endif
             dialogRef = NULL;
         }
     } else if (isOpen == SAVE_FILE) {
         err = NavCreatePutFileDialog(&diagOptions, 'TEXT', 'WIsH', 
                 openFileEventUPP, NULL, &dialogRef);
         if (err!=noErr){
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"NavCreatePutFileDialog failed, %d\n", err );
+#endif
             dialogRef = NULL;
         }
     } else if (isOpen == CHOOSE_FOLDER) {
         err = NavCreateChooseFolderDialog(&diagOptions, openFileEventUPP, 
                 openFileFilterUPP, NULL, &dialogRef);
         if (err!=noErr){
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"NavCreateChooseFolderDialog failed, %d\n", err );
+#endif
             dialogRef = NULL;
         }
     }
@@ -843,10 +853,14 @@ NavServicesGetFile(
         }
         
         if ((err = NavDialogRun(dialogRef)) != noErr ){
+#ifdef TK_MAC_DEBUG
             fprintf(stderr,"NavDialogRun failed, %d\n", err );
+#endif
         } else {
             if ((err = NavDialogGetReply(dialogRef, &theReply)) != noErr) {
+#ifdef TK_MAC_DEBUG
                 fprintf(stderr,"NavGetReply failed, %d\n", err );
+#endif
             }
         }
     }
@@ -881,15 +895,19 @@ NavServicesGetFile(
                 if (err == noErr) {
                     if ((err = AEGetDescData(&resultDesc, &fsRef, sizeof(fsRef))) 
                             != noErr ) {
+#ifdef TK_MAC_DEBUG
                         fprintf(stderr,"AEGetDescData failed %d\n", err );
+#endif
                     } else {
-                        if (err = FSRefMakePath(&fsRef, (unsigned char*) pathPtr, 1024) ) {
+                        if ((err = FSRefMakePath(&fsRef, (unsigned char*) pathPtr, 1024))) {
+#ifdef TK_MAC_DEBUG
                             fprintf(stderr,"FSRefMakePath failed, %d\n", err );
+#endif
                         } else {
                             if (isOpen == SAVE_FILE) {
                                 CFStringRef saveNameRef;
                                 char saveName [1024];
-                                if (saveNameRef = NavDialogGetSaveFileName(dialogRef)) {
+                                if ((saveNameRef = NavDialogGetSaveFileName(dialogRef))) {
                                     if (CFStringGetCString(saveNameRef, saveName, 
                                             1024, kCFStringEncodingUTF8)) {
                                         if (strlen(pathPtr) + strlen(saveName) < 1023) {
@@ -897,13 +915,19 @@ NavServicesGetFile(
                                             strcat(pathPtr, saveName);
                                             pathValid = 1;
                                         } else {
+#ifdef TK_MAC_DEBUG
                                             fprintf(stderr, "Path name too long\n");                                        
+#endif
                                         }
                                     } else {
+#ifdef TK_MAC_DEBUG
                                         fprintf(stderr, "CFStringGetCString failed\n");
+#endif
                                     }
                                 } else {
+#ifdef TK_MAC_DEBUG
                                     fprintf(stderr, "NavDialogGetSaveFileName failed\n");
+#endif
                                 }
                             } else {
                                 pathValid = 1;
@@ -1005,7 +1029,7 @@ OpenFileFilterProc(
                         fileName[len] = '\0';
                         fileNamePtr = (unsigned char*) fileName;
 
-                    } else if (theItem->descriptorType = typeFSRef) {
+                    } else if ((theItem->descriptorType = typeFSRef)) {
                         OSStatus err;
                         FSRef *theRef = (FSRef *) *theItem->dataHandle;
                         HFSUniStr255 uniFileName;
@@ -1080,7 +1104,7 @@ OpenEventProc(
         chosenItem = (NavMenuItemSpec *) callBackParams->eventData.eventDataParms.param;
         ofd->curType = chosenItem->menuType;
     } else if (callBackSelector == kNavCBAdjustRect 
-            || callBackSelector & otherEvent != 0) { 
+            || (callBackSelector & otherEvent) != 0) { 
         while (Tcl_DoOneEvent(TCL_IDLE_EVENTS
                 | TCL_DONT_WAIT 
                 | TCL_WINDOW_EVENTS)) {
@@ -1248,7 +1272,7 @@ TkAboutDlg()
     DisposeDialog(aboutDlog);
     aboutDlog = NULL;
         
-    SelectWindow(FrontNonFloatingWindow());
+    SelectWindow(ActiveNonFloatingWindow());
 
     return;
 }
@@ -1292,7 +1316,7 @@ Tk_MessageBoxObjCmd(
     int                     defaultNativeButtonIndex;    /* 1, 2, 3: right to left. */
     int                     typeIndex;
     int                     i;
-    int                     indexDefaultOption;
+    int                     indexDefaultOption = 0;
     int                     result = TCL_OK;
     
     static CONST char *movableAlertStrings[] = {
