@@ -49,6 +49,8 @@
  *      acting in its behalf permission to use and distribute the
  *      software in accordance with the terms specified in this
  *      license.
+ *
+ * RCS: @(#) $Id: tkMacOSXKeyEvent.c,v 1.6.2.6 2005/11/27 02:36:46 das Exp $
  */
 
 #include "tkMacOSXInt.h"
@@ -59,7 +61,7 @@ typedef struct {
     WindowRef   whichWindow;
     Point       global;
     Point       local;
-    int         state;
+    unsigned int state;
     unsigned char ch;
     UInt32      keyCode;
     UInt32      keyModifiers;
@@ -96,7 +98,9 @@ static int GenerateKeyEvent (
         const UniChar * chars, int numChars);
 
 static int GetKeyboardLayout (
-        Ptr * resource);
+        Ptr * resourcePtr, TextEncoding * encodingPtr);
+static TextEncoding GetKCHREncoding(
+        ScriptCode script, SInt32 layoutid);
 
 static int KeycodeToUnicodeViaUnicodeResource(
         UniChar * uniChars, int maxChars,
@@ -107,7 +111,7 @@ static int KeycodeToUnicodeViaUnicodeResource(
 
 static int KeycodeToUnicodeViaKCHRResource(
         UniChar * uniChars, int maxChars,
-        Ptr kchr,
+        Ptr kchr, TextEncoding encoding,
         EventKind eKind,
         UInt32 keycode, UInt32 modifiers,
         UInt32 * deadKeyStatePtr);
@@ -129,7 +133,8 @@ static int KeycodeToUnicodeViaKCHRResource(
  *----------------------------------------------------------------------
  */
 
-int TkMacOSXProcessKeyboardEvent(
+int
+TkMacOSXProcessKeyboardEvent(
         TkMacOSXEvent * eventPtr, 
         MacEventStatus * statusPtr)
 {
@@ -138,10 +143,8 @@ int TkMacOSXProcessKeyboardEvent(
     static UniChar savedChar = 0;
     OSStatus     status;
     KeyEventData keyEventData;
-#if 0
     MenuRef   menuRef;
     MenuItemIndex menuItemIndex;
-#endif
     int eventGenerated;
     UniChar uniChars[5]; /* make this larger, if needed */
     UInt32 uniCharsLen = 0;
@@ -150,51 +153,46 @@ int TkMacOSXProcessKeyboardEvent(
         statusPtr->err = 1;
         return false;
     }
-
-#if 0
-    /*
-     * This block of code seems like a good idea, to trap
-     * key-bindings which point directly to menus, but it
-     * has a number of problems:
-     * (1) when grabs are present we definitely don't want
-     * to do this.
-     * (2) Tk's semantics define accelerator keystrings in
-     * menus as a purely visual adornment, and require that
-     * the developer create separate bindings to trigger
-     * them.  This breaks those semantics.  (i.e. Tk will
-     * behave differently on Aqua to the behaviour on Unix/Win).
-     * (3) Tk's bindings depend on the current window's bindtags,
-     * which may be completely different to what happens to be
-     * in some global menu (agreed, it shouldn't be that different,
-     * but it often is).
-     * 
-     * While a better middleground might be possible, the best, most
-     * compatible, approach at present is to disable this block.
-     */
-    if (IsMenuKeyEvent(NULL, eventPtr->eventRef, 
-            kNilOptions, &menuRef, &menuItemIndex)) {
-        int    oldMode;
-        MenuID menuID;
-        KeyMap theKeys;
-        int    selection;
-        
-        menuID = GetMenuID(menuRef);
-        selection = (menuID << 16) | menuItemIndex;
     
-        GetKeys(theKeys);
-        oldMode = Tcl_SetServiceMode(TCL_SERVICE_ALL);
-        TkMacOSXClearMenubarActive();
- 
-        /*
-         * Handle -postcommand
-         */
-         
-        TkMacOSXPreprocessMenu();
-        TkMacOSXHandleMenuSelect(selection, theKeys[1] & 4);
-        Tcl_SetServiceMode(oldMode);
-        return 0; /* TODO: may not be on event on queue. */
+    /*
+     * Because of the way that Tk operates, we can't in general funnel menu
+     * accelerators through IsMenuKeyEvent.  Tk treats accelerators as mere
+     * decoration, and the user has to install bindings to get them to fire.
+     *
+     * However, the only way to trigger the Hide & Hide Others functions
+     * is by invoking the Menu command for Hide.  So there is no nice way to
+     * provide a Tk command to hide the app which would be available for a
+     * binding.  So I am going to hijack Command-H and Command-Shift-H 
+     * here, and run the menu commands.  Since the HI Guidelines explicitly
+     * reserve these for Hide, this isn't such a bad thing.  Also, if you do
+     * rebind Command-H to another menu item, Hide will lose its binding.
+     * 
+     * Note that I don't really do anything at this point, 
+     * I just mark stopProcessing as 0 and return, and then the
+     * RecieveAndProcessEvent code will dispatch the event to the default
+     * handler.
+     */
+
+    if ((eventPtr->eKind == kEventRawKeyDown 
+            || eventPtr->eKind == kEventRawKeyRepeat) 
+            && IsMenuKeyEvent(NULL, eventPtr->eventRef, 
+            kMenuEventQueryOnly, &menuRef, &menuItemIndex)) {
+        MenuCommand menuCmd;
+        
+        GetMenuItemCommandID (menuRef, menuItemIndex, &menuCmd);
+        
+        switch (menuCmd) {
+            case kHICommandHide:
+            case kHICommandHideOthers:
+            case kHICommandShowAll:
+            case kHICommandPreferences:
+                statusPtr->stopProcessing = 0;
+                return 0; /* TODO: may not be on event on queue. */
+                break;
+            default:
+                break;
+        }
     }
-#endif
 
     status = GetEventParameter(eventPtr->eventRef, 
             kEventParamKeyMacCharCodes,
@@ -202,7 +200,9 @@ int TkMacOSXProcessKeyboardEvent(
             sizeof(keyEventData.ch), NULL,
             &keyEventData.ch);
     if (status != noErr) {
+#ifdef TK_MAC_DEBUG
         fprintf (stderr, "Failed to retrieve KeyMacCharCodes\n");
+#endif
         statusPtr->err = 1;
         return false;
     } 
@@ -212,7 +212,9 @@ int TkMacOSXProcessKeyboardEvent(
             sizeof(keyEventData.keyCode), NULL,
             &keyEventData.keyCode);
     if (status != noErr) {
+#ifdef TK_MAC_DEBUG
         fprintf (stderr, "Failed to retrieve KeyCode\n");
+#endif
         statusPtr->err = 1;
         return false;
     }
@@ -222,7 +224,9 @@ int TkMacOSXProcessKeyboardEvent(
             sizeof(keyEventData.keyModifiers), NULL,
             &keyEventData.keyModifiers);
     if (status != noErr) {
+#ifdef TK_MAC_DEBUG
         fprintf (stderr, "Failed to retrieve KeyModifiers\n");
+#endif
         statusPtr->err = 1;
         return false;
     }
@@ -365,7 +369,7 @@ GenerateKeyEvent(
 
     } else {
 
-        for (i=0; i<numChars; ++i) {
+        for (i = 0; i < numChars; ++i) {
 
             /*
              * Encode one char in the trans_chars array that was already
@@ -399,9 +403,11 @@ GenerateKeyEvent(
                     Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
                     break;
                 default:
+#ifdef TK_MAC_DEBUG
                     fprintf (stderr,
                             "GenerateKeyEvent(): Invalid parameter eKind %d\n",
                             (int) eKind);
+#endif
                     return -1;
             } 
         }
@@ -430,16 +436,19 @@ GenerateKeyEvent(
 static int 
 InitKeyData(KeyEventData * keyEventDataPtr)
 {
+    int x, y;
     memset (keyEventDataPtr, 0, sizeof(*keyEventDataPtr));
 
-    keyEventDataPtr->whichWindow = FrontNonFloatingWindow();
+    keyEventDataPtr->whichWindow = ActiveNonFloatingWindow();
     if (keyEventDataPtr->whichWindow == NULL) {
         return false;
     }
-    GetMouse(&keyEventDataPtr->local);
-    keyEventDataPtr->global = keyEventDataPtr->local;
-    LocalToGlobal(&keyEventDataPtr->global);
-    keyEventDataPtr->state = TkMacOSXButtonKeyState();
+    XQueryPointer(NULL, None, NULL, NULL, &x, &y, 
+            NULL, NULL, &keyEventDataPtr->state);
+    keyEventDataPtr->global.v = x;
+    keyEventDataPtr->global.v = y;
+    keyEventDataPtr->local = keyEventDataPtr->global;
+    GlobalToLocal(&keyEventDataPtr->local);
 
     return true;
 }
@@ -506,13 +515,17 @@ InitKeyEvent(
     tkwin = Tk_IdToWindow(dispPtr->display, window);
     
     if (tkwin == NULL) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"tkwin == NULL, %d\n", __LINE__);
+#endif
         return -1;
     }
     
     tkwin = (Tk_Window) ((TkWindow *) tkwin)->dispPtr->focusPtr;
     if (tkwin == NULL) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"tkwin == NULL, %d\n", __LINE__);
+#endif
         return -1;
     }
 
@@ -542,6 +555,22 @@ InitKeyEvent(
 }
 
 
+/*
+ * If we have old headers, we need to define these types and constants
+ * ourself.  We use preprocessor macros instead of enums and typedefs,
+ * because macros work even in case of version misunderstandings, while
+ * duplicate enums and typedefs would give errrors.
+ */
+
+#if !defined(MAC_OS_X_VERSION_10_2) || \
+    (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_2)
+#define KeyboardLayoutRef Ptr
+#define KeyboardLayoutPropertyTag UInt32
+#define kKLKCHRData 0
+#define kKLuchrData 1
+#define kKLIdentifier 2
+#endif
+
 /*
  *----------------------------------------------------------------------
  *
@@ -549,13 +578,14 @@ InitKeyEvent(
  *
  *      Queries the OS for a pointer to a keyboard resource.
  *
- *      This function works with the keyboard layout switch menu that
- *      we have in 10.2.
+ *      This function works with the keyboard layout switch menu.  It uses
+ *      Keyboard Layout Services, where available.
  *
  * Results:
- *      1 if there is returned a Unicode 'uchr' resource in
- *      "*resource", 0 if it is a classic 'KCHR' resource.  A pointer
- *      to the actual resource data goes into *resource.
+ *      1 if there is returned a Unicode 'uchr' resource in *resourcePtr, 0
+ *      if it is a classic 'KCHR' resource.  A pointer to the actual resource
+ *      data goes into *resourcePtr.  If the resource is a 'KCHR' resource,
+ *      the corresponding Mac encoding goes into *encodingPtr.
  *
  * Side effects:
  *      Sets some internal static variables.
@@ -564,51 +594,260 @@ InitKeyEvent(
  */
 
 static int
-GetKeyboardLayout (Ptr * resource)
+GetKeyboardLayout (Ptr * resourcePtr, TextEncoding * encodingPtr)
 {
-    static Boolean initialized = false;
-    static SInt16 lastKeyLayoutID = -1;
-    static Handle uchrHnd = NULL;
-    static Handle KCHRHnd = NULL;
+    static KeyboardLayoutRef lastLayout = NULL;
+    static SInt32 lastLayoutId;
+    static TextEncoding lastEncoding = kTextEncodingMacRoman;
+    static Ptr uchr = NULL;
+    static Ptr KCHR = NULL;
+    static Handle handle = NULL;
 
-    SInt16 keyScript;
-    SInt16 keyLayoutID;
+    int hasLayoutChanged = false;
+    KeyboardLayoutRef currentLayout = NULL;
+    SInt32 currentLayoutId = 0;
+    ScriptCode currentKeyScript;
 
-    keyScript = GetScriptManagerVariable(smKeyScript);
-    keyLayoutID = GetScriptVariable(keyScript,smScriptKeys);
+    /*
+     * Several code branches need this information.
+     */
 
-    if (!initialized || (lastKeyLayoutID != keyLayoutID)) {
-        initialized = true;
-        deadKeyStateUp = deadKeyStateDown = 0;
-        lastKeyLayoutID = keyLayoutID;
-        uchrHnd = GetResource('uchr',keyLayoutID);
-        if (NULL == uchrHnd) {
-            KCHRHnd = GetResource('KCHR',keyLayoutID);
+    currentKeyScript = GetScriptManagerVariable(smKeyScript);
+
+    TkMacOSXInitNamedSymbol(HIToolbox, OSStatus, KLGetCurrentKeyboardLayout,
+            KeyboardLayoutRef*);
+    TkMacOSXInitNamedSymbol(HIToolbox, OSStatus, KLGetKeyboardLayoutProperty,
+            KeyboardLayoutRef, KeyboardLayoutPropertyTag, const void**);
+    if (KLGetCurrentKeyboardLayout && KLGetKeyboardLayoutProperty) {
+
+        /*
+         * Use the Keyboard Layout Services (these functions only exist since
+         * 10.2).
+         */
+
+        KLGetCurrentKeyboardLayout(&currentLayout);
+
+        if (currentLayout != NULL) {
+
+            /*
+             * The layout pointer could in theory be the same for different
+             * layouts, only the id gives us the information that the
+             * keyboard has actually changed.  OTOH the layout object can
+             * also change and it could still be the same layoutid.
+             */
+
+            KLGetKeyboardLayoutProperty(currentLayout, kKLIdentifier,
+                    (const void**)&currentLayoutId);
+
+            if ((lastLayout != currentLayout)
+                    || (lastLayoutId != currentLayoutId)) {
+
+#ifdef TK_MAC_DEBUG
+                fprintf (stderr, "GetKeyboardLayout(): Use KLS\n");
+#endif
+
+                hasLayoutChanged = true;
+
+                /*
+                 * Reinitialize all relevant variables.
+                 */
+
+                lastLayout = currentLayout;
+                lastLayoutId = currentLayoutId;
+                uchr = NULL;
+                KCHR = NULL;
+
+                if ((KLGetKeyboardLayoutProperty(currentLayout,
+                                        kKLuchrData, (const void**)&uchr)
+                                == noErr)
+                        && (uchr != NULL)) {
+                    /* done */
+                } else if ((KLGetKeyboardLayoutProperty(currentLayout,
+                                        kKLKCHRData, (const void**)&KCHR)
+                                == noErr)
+                        && (KCHR != NULL)) {
+                    /* done */
+                }
+            }
         }
-        if ((NULL == uchrHnd) && (NULL == KCHRHnd)) {
-            initialized = false;
-            fprintf (stderr,
-                    "GetKeyboardLayout(): "
-                    "Can't get a keyboard layout for layout %d "
-                    "(error code %d)?\n",
-                    (int) keyLayoutID, (int) ResError());
-            *resource = (Ptr) GetScriptManagerVariable(smKCHRCache);
-            fprintf (stderr,
-                    "GetKeyboardLayout(): Trying the cache: %p\n",
-                    *resource);
-            return 0;
+
+    } else {
+
+        /*
+         * Use the classic approach as shown in Apple code samples, loading
+         * the keyboard resources directly.  This is broken for 10.3 and
+         * possibly already in 10.2.
+         */
+
+        currentLayoutId = GetScriptVariable(currentKeyScript,smScriptKeys);
+
+        if ((lastLayout == NULL) || (lastLayoutId != currentLayoutId)) {
+
+#ifdef TK_MAC_DEBUG
+            fprintf (stderr, "GetKeyboardLayout(): Use GetResource()\n");
+#endif
+
+            hasLayoutChanged = true;
+
+            /*
+             * Reinitialize all relevant variables.
+             */
+
+            lastLayout = (KeyboardLayoutRef)-1; 
+            lastLayoutId = currentLayoutId;
+            uchr = NULL;
+            KCHR = NULL;
+
+            /*
+             * Get the new layout resource in the classic way.
+             */
+
+            if (handle != NULL) {
+                HUnlock(handle);
+            }
+            
+            if ((handle = GetResource('uchr',currentLayoutId)) != NULL) {
+                HLock(handle);
+                uchr = *handle;
+            } else if ((handle = GetResource('KCHR',currentLayoutId)) != NULL) {
+                HLock(handle);
+                KCHR = *handle;
+            }
         }
     }
 
-    if (NULL != uchrHnd) {
-        *resource = *uchrHnd;
+    if (hasLayoutChanged) {
+
+#ifdef TK_MAC_DEBUG
+        if (KCHR != NULL) {
+            fprintf (stderr, "GetKeyboardLayout(): New 'KCHR' layout %d\n",
+                    (int) (short) currentLayoutId);
+        } else if (uchr != NULL) {
+            fprintf (stderr, "GetKeyboardLayout(): New 'uchr' layout %d\n",
+                    (int) (short) currentLayoutId);
+        } else {
+            fprintf (stderr, "GetKeyboardLayout(): Use cached layout "
+                    "(should have been %d)\n",
+                    (int) (short) currentLayoutId);
+        }
+#endif
+
+        deadKeyStateUp = deadKeyStateDown = 0;
+
+        /*
+         * If we did get a new 'KCHR', compute its encoding and put it into
+         * lastEncoding.
+         *
+         * If we didn't get a new 'KCHR' and if we have no 'uchr' either, get
+         * some 'KCHR' from the OS cache and leave lastEncoding at its
+         * current value.  This should better not happen, it doesn't really
+         * work.
+         */
+
+        if (KCHR != NULL) {
+            lastEncoding = GetKCHREncoding(currentKeyScript, currentLayoutId);
+#ifdef TK_MAC_DEBUG
+            fprintf (stderr, "GetKeyboardLayout(): New 'KCHR' encoding %lu "
+                    "(%lu + 0x%lX)\n",
+                    lastEncoding, lastEncoding & 0xFFFFL,
+                    lastEncoding & ~0xFFFFL);
+#endif
+        } else if (uchr == NULL) {
+            KCHR = (Ptr) GetScriptManagerVariable(smKCHRCache);
+        }
+    }
+
+    if (uchr != NULL) {
+        *resourcePtr = uchr;
         return 1;
     } else {
-        *resource = *KCHRHnd;
+        *resourcePtr = KCHR;
+        *encodingPtr = lastEncoding;
         return 0;
     }
 }
 
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * GetKCHREncoding --
+ *
+ *      Upgrade a WorldScript code to a TEC encoding based on the keyboard
+ *      layout id.
+ *
+ * Results:
+ *      The TEC code that corresponds best to the combination of WorldScript
+ *      code and 'KCHR' id.
+ *
+ * Side effects:
+ *      None.
+ *
+ * Rationale and Notes:
+ *      WorldScript codes are sometimes not unique encodings.  E.g. Icelandic
+ *      uses script smRoman (0), but the actual encoding is
+ *      kTextEncodingMacIcelandic (37).  ftp://ftp.unicode.org/Public
+ *      /MAPPINGS/VENDORS/APPLE/README.TXT has a good summary of these
+ *      variants.  So we need to upgrade the script to an encoding with
+ *      GetTextEncodingFromScriptInfo().
+ *
+ *      'KCHR' ids are usually region codes (see the comments in Script.h).
+ *      Where they are not, we get a paramErr from the OS function and have
+ *      appropriate fallbacks.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static TextEncoding
+GetKCHREncoding(ScriptCode script, SInt32 layoutid)
+{
+    RegionCode region = layoutid;
+    TextEncoding encoding = script;
+
+    if (GetTextEncodingFromScriptInfo(script, kTextLanguageDontCare, region,
+                    &encoding) == noErr) {
+        return encoding;
+    }
+
+    /*
+     * GetTextEncodingFromScriptInfo() doesn't know about more exotic
+     * layouts.  This provides a fallback for good measure.  In an ideal
+     * world, exotic layouts would always provide a 'uchr' resource anyway,
+     * so we wouldn't need this.
+     *
+     * We can add more keyboard layouts, if we get actual complaints.  Farsi
+     * or other Celtic/Gaelic layouts would be candidates.
+     */
+
+    switch (layoutid) {
+
+        /*
+         * Icelandic and Faroese (planned).  These layouts are sold by Apple
+         * Iceland for legacy applications.
+         */
+
+        case 1800: case 1821:
+            return kTextEncodingMacIcelandic;
+
+        /*
+         * Irish and Welsh.  These layouts are mentioned in <Script.h>.
+         *
+         * FIXME: This may have to be kTextEncodingMacGaelic instead, but I
+         * can't locate layouts of this type for testing.
+         */
+
+        case 581: case 779:
+            return kTextEncodingMacCeltic;
+    }
+    
+    /*
+     * The valid script codes are also the valid default encoding codes, so
+     * if nothing else helps, fall back on those.
+     */
+
+    return script;
+}
+
 
 /*
  *----------------------------------------------------------------------
@@ -672,10 +911,12 @@ KeycodeToUnicodeViaUnicodeResource(
             action = kUCKeyActionAutoKey;
             break;
         default:
+#ifdef TK_MAC_DEBUG
             fprintf (stderr,
                     "KeycodeToUnicodeViaUnicodeResource(): "
                     "Invalid parameter eKind %d\n",
                     (int) eKind);
+#endif
             return 0;
     }
 
@@ -700,7 +941,9 @@ KeycodeToUnicodeViaUnicodeResource(
     *deadKeyStatePtr = 0; 
 
     if (noErr != status) {
+#ifdef TK_MAC_DEBUG
         fprintf(stderr,"UCKeyTranslate failed: %d", (int) status);
+#endif
         actuallength = 0;
     }
 
@@ -733,7 +976,7 @@ KeycodeToUnicodeViaUnicodeResource(
 static int
 KeycodeToUnicodeViaKCHRResource(
         UniChar * uniChars, int maxChars,
-        Ptr kchr,
+        Ptr kchr, TextEncoding encoding,
         EventKind eKind,
         UInt32 keycode, UInt32 modifiers,
         UInt32 * deadKeyStatePtr)
@@ -802,19 +1045,34 @@ KeycodeToUnicodeViaKCHRResource(
     if (macStrLen <= 0) {
         return 0;
     } else {
+
         /*
          * Use the CFString conversion routines.  This is the easiest and
          * most compatible way to get from an 8-bit string and a MacOS script
          * code to a Unicode string.
+         *
+         * FIXME: The system ships with an Irish 'KCHR' but without the
+         * corresponding macCeltic encoding, which triggers the error below.
+         * Tcl doesn't have the macCeltic encoding either right now, so until
+         * we get that, we can just as well stick to this code.  The right
+         * fix would be to use the Tcl encodings and add macCeltic and
+         * probably others there.  Suitable Unicode data files for the
+         * missing encodings are available from www.evertype.com.
          */
-
+        
         CFStringRef cfString;
         int uniStrLen;
 
         cfString = CFStringCreateWithCStringNoCopy(
-                NULL, macStr,
-                GetScriptManagerVariable(smKeyScript),
-                kCFAllocatorNull);
+            NULL, macStr, encoding, kCFAllocatorNull);
+        if (cfString == NULL) {
+#ifdef TK_MAC_DEBUG
+            fprintf(stderr, "CFString: Can't convert with encoding %d\n",
+                    (int) encoding);
+#endif
+            return 0;
+        }
+
         uniStrLen = CFStringGetLength(cfString);
         if (uniStrLen > maxChars) {
             uniStrLen = maxChars;
@@ -860,17 +1118,18 @@ TkMacOSXKeycodeToUnicode(
         UInt32 * deadKeyStatePtr)
 {
     Ptr resource = NULL;
+    TextEncoding encoding;
     int len;
 
 
-    if (GetKeyboardLayout(&resource)) {
+    if (GetKeyboardLayout(&resource,&encoding)) {
         len = KeycodeToUnicodeViaUnicodeResource(
-                uniChars, maxChars, resource, eKind,
-                keycode, modifiers, deadKeyStatePtr);
+            uniChars, maxChars, resource, eKind,
+            keycode, modifiers, deadKeyStatePtr);
     } else {
         len = KeycodeToUnicodeViaKCHRResource(
-                uniChars, maxChars, resource, eKind,
-                keycode, modifiers, deadKeyStatePtr);
+            uniChars, maxChars, resource, encoding, eKind,
+            keycode, modifiers, deadKeyStatePtr);
     }
 
     return len;
